/*
  Requires NI-VISA 2.6 or higher.

*/

#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "2400_DCIV.h"

#include "visa.h"

static ViSession defaultRM;
static ViSession instr;
static ViStatus status; 
static ViUInt32 retCount;
static ViUInt32 writeCount;
//static ViInt32	errorCode;
static ViChar	errorMessage[256];
static unsigned char buffer[100];
static char stringinput[512];



static int panelHandle;

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "2400_DCIV.uir", mainPanel)) < 0)
		return -1;
	DisplayPanel (panelHandle);
	RunUserInterface ();
	DiscardPanel (panelHandle);
	return 0;
}

int CVICALLBACK OnOpen (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:

    /*
     * First we must call viOpenDefaultRM to get the resource manager
     * handle.  We will store this handle in defaultRM.
     */
   status=viOpenDefaultRM (&defaultRM);
   if (status < VI_SUCCESS)
   {
      // need an error handler.....
      
      break;  // exit from here
    } 
                                                                         
    /*
     * Now we will open a VISA session to a device at Primary Address 24.
     *
     * We must use the handle from viOpenDefaultRM and we must   
     * also use a string that indicates which instrument to open.  This
     * is called the instrument descriptor.  The format for this string
     * can be found in the NI-VISA User Manual.
     * After opening a session to the device, we will get a handle to 
     * the instrument which we will use in later VISA functions.  
     * The two parameters in this function which are left blank are
     * reserved for future functionality.  These two parameters are 
     * given the value VI_NULL.
     *
     * This example will also work for serial instruments by changing 
     * the instrument descriptor from GPIB0::24::INSTR to ASRL1::INSTR or
     * the necessary descriptor for your instrument.
     */
     
     // read in the VISA resource from the Panel
     GetCtrlVal(mainPanel, mainPanel_ResourceName, stringinput);
     // Set Build Options....Debugging Level set to 'no run-time checking' so that
     // CVI does not pop-up runtime errors....let our error checking do the job instead...
   if ((status = viOpen (defaultRM,stringinput, VI_NULL, VI_NULL, &instr)) < 0)
   {
      status = viStatusDesc(defaultRM, status, errorMessage); // pass in status from previous op
        // set our identify control equal to the response
      SetCtrlVal (mainPanel, mainPanel_errStatus, errorMessage);
      break;  // exit from here
    }
   else
   {  // no error.  Do some other stuff like set the time out and
     // ask instrument to identify itself.
  
    /*
     * Set timeout value to 2000 milliseconds (2 seconds).
     */ 
   status = viSetAttribute (instr, VI_ATTR_TMO_VALUE, 2000);
  
    /*
     * At this point we now have a session open to the instrument 
     * defined by the Resource Name.  We can use this session handle to write 
     * ASCII (SCPI) commands to the instrument.
     */
     
     strcpy(stringinput,"*IDN?");
     status = viWrite (instr, (ViBuf)stringinput, strlen(stringinput), &writeCount);
   if (status < VI_SUCCESS)  
   {
      // need error handler....
      status = viStatusDesc(defaultRM, status, errorMessage);
        // set our identify control equal to the response
      SetCtrlVal (mainPanel, mainPanel_errStatus, errorMessage);
      break;  // exit from here
    }
  
    /*
     * Now we will attempt to read back a response from the device to
     * the identification query that was sent.  We will use the viRead
     * function to acquire the data.  We will try to read back 100 bytes.
     * After the data has been read the response is displayed.
     */
   status = viRead (instr, buffer, 100, &retCount);
   if (status < VI_SUCCESS) 
   {
      // need error handler.....
      status = viStatusDesc(defaultRM, status, errorMessage);
        // set our identify control equal to the response
      SetCtrlVal (mainPanel, mainPanel_errStatus, errorMessage);
      break;  // exit from here
   }
   else
   {
      //there was no error reading...display the results and enable other buttons
      
      // set our identify control equal to the response
      SetCtrlVal (mainPanel, mainPanel_Identify, buffer);
      SetCtrlAttribute (mainPanel, mainPanel_Close, ATTR_DIMMED,0);  // enable the Close button
      SetCtrlAttribute (mainPanel, mainPanel_Measure, ATTR_DIMMED,0);  // enable the Measure button 
      // in case there are old error strings displayed....call again with our successful outcome
      status = viStatusDesc(defaultRM, status, errorMessage);
        // set our identify control equal to the response...will display successful result string
      SetCtrlVal (mainPanel, mainPanel_errStatus, errorMessage);
  
      }
   } // end if
 
	break;
		}
	return 0;
}

int CVICALLBACK OnMeasure (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			// use SCPI commands to have the 2400 source 1 volt at 100mA compliance
			// and then measure the current.  Display the value in MeasuredValue control.
			
	 strcpy(stringinput,":Sour:Volt:Range:Auto On;:Sour:Volt 1");
     status = viWrite (instr, (ViBuf)stringinput, strlen(stringinput), &writeCount);
     
     strcpy(stringinput,":Sens:Curr:Prot 0.1");
     status = viWrite (instr, (ViBuf)stringinput, strlen(stringinput), &writeCount);
     
     strcpy(stringinput,":OUTPUT On;:READ?");
     status = viWrite (instr, (ViBuf)stringinput, strlen(stringinput), &writeCount);
     
	 status = viRead (instr, buffer, 100, &retCount);		
	 SetCtrlVal (mainPanel, mainPanel_MeasuredValue, buffer);
	 
	 strcpy(stringinput,":OUTPUT Off");
     status = viWrite (instr, (ViBuf)stringinput, strlen(stringinput), &writeCount);
     
     // error checking should have been performed, but for clarity of code it was
     // not shown here
	
			break;
		}					 
	return 0;
}

int CVICALLBACK OnClose (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
		    // take instrument out of REMOTE operation
		    status = viGpibControlREN(instr,VI_GPIB_REN_DEASSERT_GTL);
		    // close down our VISA session
		    status = viClose(instr);
            status = viClose(defaultRM);
			QuitUserInterface (0);
			break;
		}
	return 0;
}



